#include "pub.h"
#include <filesystem>

namespace fs = std::filesystem;

CUtil::CUtil() = default;
CUtil::~CUtil() = default;

bool CUtil::read_txt(const std::string& path, std::string& content)
{
    std::ifstream file(path);
    if (!file.is_open()) {
        return false;
    }
    std::string tp((std::istreambuf_iterator<char>(file)), (std::istreambuf_iterator<char>()));
    file.close();
    content = std::move(tp);
    return true;
}

bool CUtil::save_txt(const std::string& path, const std::string& content)
{
    std::ofstream file(path, std::ios::out);
    if (!file.is_open()) {
        return false;
    }
    file << content;
    file.close();
    return true;
}

std::string CUtil::replace(const std::string& content, const std::string& from, const std::string& to)
{
    std::string tp(content);
    if (from.empty()) {
        return content;
    }
    std::size_t start_pos = 0;
    while ((start_pos = tp.find(from, start_pos)) != std::string::npos) {
        tp.replace(start_pos, from.length(), to);
        start_pos += to.length();
    }
    return tp;
}

std::string CUtil::get_home()
{
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
    char* value = nullptr;
    std::size_t len = 0;
    auto err = _dupenv_s(&value, &len, "USERPROFILE");
    if (err == 0 && value != nullptr) {
        std::string ret(value);
        free(value);
        return ret;
    }
    else {
        return "";
    }
#else
    const char *homedir = getenv("HOME");
    if (homedir) {
        return std::string(homedir);
    }
    return "";
#endif
}


std::string CUtil::get_keybindings_file()
{
    std::string path;
    std::string home_dir = get_home();
    fs::path ret(home_dir);
#ifdef _WIN32
    ret.append("AppData/Roaming/Code/User");
#elif defined(__clang__) && defined(__APPLE__)
    ret.append("Library/Application Support/Code/User");
#else
    ret.append(".config/Code/User");
#endif
    path = ret.append("keybindings.json").string();
    return path;
}